==> fred_verifier-0.1.0-py3-none-any.whl: verifier/tests/test_client.py <==
from datetime import datetime, timedelta
from typing import Sequence, Tuple
from unittest import IsolatedAsyncioTestCase, TestCase
from unittest.mock import sentinel

import pytz
from aioitertools import iter as aiter
from fred_api.verification.common_types_pb2 import (
    Event as EventMessage,
    PlaceAddress,
    VerificationInfo,
    VerificationStatus as VerificationEnum,
)
from fred_api.verification.pagination_pb2 import PaginationReply
from fred_api.verification.service_verification_grpc_pb2 import (
    CancelReply,
    CancelRequest,
    CreateReply,
    CreateRequest,
    GetContactRepresentativeReply,
    GetContactRepresentativeRequest,
    GetMethodsReply,
    GetMethodsRequest,
    GetReply,
    GetRequest,
    ListReply,
    ListRequest,
    MarkVerifiedReply,
    MarkVerifiedRequest,
    SetContactRepresentativeReply,
    SetContactRepresentativeRequest,
    SetMethodReply,
    SetMethodRequest,
    VerifyTokenReply,
    VerifyTokenRequest,
)
from frgal.utils import AsyncTestClientMixin, make_awaitable
from regal import Address, ContactId

from verifier.client import VerificationClient, VerificationDecoder
from verifier.common import (
    ContactData,
    ContactRepresentative,
    Event,
    PaginatedList,
    Pagination,
    Verification,
    VerificationId,
    VerificationStatus,
)

DATETIME = datetime(2022, 1, 1, tzinfo=pytz.utc)


def get_event(
    event_datetime: datetime = DATETIME,
    type: str = "notification",
    details: dict = None,
) -> Tuple[EventMessage, Event]:
    """Get tuple of matching (gRPC message, Event instance)."""
    message = EventMessage()
    message.event_datetime.FromDatetime(event_datetime)
    message.type = type
    for key, value in (details or {}).items():
        message.details[key] = value

    event = Event(event_datetime=event_datetime, type=type, details=details or {})
    return message, event


def get_verification(
    verification_id: str = "v:1",
    contact_id: str = "c:1",
    method: str = None,
    create_datetime: datetime = DATETIME,
    close_datetime: datetime = None,
    status: VerificationStatus = VerificationStatus.CREATED,
    next_event: Event = None,
    events: Sequence[Event] = None,
) -> Tuple[VerificationInfo, Verification]:
    """Get tuple of matching (gRPC message, Verification instance)."""
    message = VerificationInfo()
    message.verification_id.value = verification_id
    message.contact_id.value = contact_id
    if method:
        message.method = method
    message.create_datetime.FromDatetime(create_datetime)
    if close_datetime:
        message.close_datetime.FromDatetime(close_datetime)
    message.status = VerificationEnum.Value(status)

    if next_event:
        next_event_message, _ = get_event(**next_event.dict())
        message.next_event.CopyFrom(next_event_message)

    for event in events or []:
        event_message, _ = get_event(**event.dict())
        subevent = message.events.add()
        subevent.CopyFrom(event_message)

    verification = Verification(
        verification_id=verification_id,
        contact_id=contact_id,
        method=method,
        create_datetime=create_datetime,
        close_datetime=close_datetime,
        status=status,
        next_event=next_event,
        events=events or [],
    )

    return (message, verification)


def get_address(
    street: Sequence[str] = ("Deck 12", "Section B"),
    city: str = "Red Dwarf",
    state_or_province: str = "",
    postal_code: str = "123",
    country_code: str = "JMC",
) -> Tuple[PlaceAddress, Address]:
    message = PlaceAddress()
    message.street[:] = street
    message.city = city
    message.state_or_province = state_or_province
    message.postal_code = postal_code
    message.country_code = country_code

    address = Address(
        street=list(street),
        city=city,
        state_or_province=state_or_province,
        postal_code=postal_code,
        country_code=country_code,
    )

    return (message, address)


class TestVerificationClient(AsyncTestClientMixin, VerificationClient):
    """Test VerificationClient."""


class VerificationDecoderTest(TestCase):
    """Test VerificationDecoder."""

    def setUp(self):
        self.decoder = VerificationDecoder()

    def test_decode_event(self):
        message, event = get_event(details={"meta": "data"})
        self.assertEqual(self.decoder.decode(message), event)

    def test_decode_event_no_details(self):
        message, event = get_event()
        self.assertEqual(self.decoder.decode(message), event)

    def test_decode_verification_empty(self):
        message, verification = get_verification()
        self.assertEqual(self.decoder.decode(message), verification)

    def test_decode_verification_full(self):
        message, verification = get_verification(
            method="eID",
            close_datetime=DATETIME + timedelta(days=1),
            status=VerificationStatus.CANCELED,
            next_event=get_event(event_datetime=DATETIME + timedelta(days=1))[1],
            events=[
                get_event(
                    event_datetime=DATETIME + timedelta(days=2),
                    details={"email": "kryten@example.org"},
                )[1]
            ],
        )
        self.assertEqual(self.decoder.decode(message), verification)

    def test_decode_pagination_empty(self):
        message = PaginationReply()
        self.assertEqual(self.decoder.decode(message), Pagination())

    def test_decode_pagination_full(self):
        message = PaginationReply()
        message.next_page_token = "gazpacho"
        message.items_left = 42
        self.assertEqual(
            self.decoder.decode(message),
            Pagination(next_page_token="gazpacho", items_left=42),
        )

    def test_decode_address(self):
        message, address = get_address()
        self.assertEqual(self.decoder.decode(message), address)


class VerificationClientTest(IsolatedAsyncioTestCase):
    """Test VerificationClient's methods."""

    def setUp(self):
        self.client = TestVerificationClient(netloc=sentinel.netloc)

    async def test_create(self):
        message, verification = get_verification()
        reply = CreateReply()
        reply.data.verification.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(await self.client.create(ContactId("c:1")), verification)

        request = CreateRequest()
        request.contact_id.value = "c:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/create",
            timeout=None,
        )

    async def test_cancel(self):
        reply = CancelReply()
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(await self.client.cancel(VerificationId("v:1")), None)

        request = CancelRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/cancel",
            timeout=None,
        )

    async def test_get(self):
        message, verification = get_verification()
        reply = GetReply()
        reply.data.verification.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(await self.client.get(VerificationId("v:1")), verification)

        request = GetRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/get",
            timeout=None,
        )

    async def test_list_empty(self):
        reply = ListReply()
        reply.data.verifications.extend([])
        self.client.mock.return_value = aiter([reply, ListReply()])

        self.assertEqual(
            await self.client.list(), PaginatedList(results=[], pagination=None)
        )

        request = ListRequest()
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/list",
            timeout=None,
        )

    async def test_list_full(self):
        message1, verification1 = get_verification(verification_id="v:1")
        message2, verification2 = get_verification(verification_id="v:2")
        reply1 = ListReply()
        reply1.data.verifications.add().CopyFrom(message1)
        reply1.data.pagination.next_page_token = "gazpacho!"
        reply1.data.pagination.items_left = 42
        reply2 = ListReply()
        reply2.data.verifications.add().CopyFrom(message2)
        self.client.mock.return_value = aiter([reply1, reply2])

        self.assertEqual(
            await self.client.list(
                contact_id=ContactId("RIMMER"),
                include_closed=True,
                page_token="gazpacho?",
                page_size=10,
            ),
            PaginatedList(
                results=[verification1, verification2],
                pagination=Pagination(next_page_token="gazpacho!", items_left=42),
            ),
        )

        request = ListRequest()
        request.contact_id.value = "RIMMER"
        request.include_closed = True
        request.pagination.page_token = "gazpacho?"
        request.pagination.page_size = 10
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/list",
            timeout=None,
        )

    async def test_get_methods(self):
        reply = GetMethodsReply()
        reply.data.methods[:] = ["eID", "letter"]
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.get_methods(VerificationId("v:1")), ["eID", "letter"]
        )

        request = GetMethodsRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/get_methods",
            timeout=None,
        )

    async def test_set_method(self):
        message, verification = get_verification()
        reply = SetMethodReply()
        reply.data.verification.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.set_method(VerificationId("v:1"), "eID"), verification
        )

        request = SetMethodRequest()
        request.verification_id.value = "v:1"
        request.method = "eID"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/set_method",
            timeout=None,
        )

    async def test_mark_verified_no_method(self):
        message, verification = get_verification()
        reply = MarkVerifiedReply()
        reply.data.verification.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.mark_verified(
                VerificationId("v:1"), verified_data={ContactData.EMAIL}
            ),
            verification,
        )

        request = MarkVerifiedRequest()
        request.verification_id.value = "v:1"
        request.email = True
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/mark_verified",
            timeout=None,
        )

    async def test_mark_verified_with_method(self):
        message, verification = get_verification()
        reply = MarkVerifiedReply()
        reply.data.verification.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.mark_verified(
                VerificationId("v:1"),
                method="eID",
                verified_data={ContactData.NAME, ContactData.BIRTHDATE},
            ),
            verification,
        )

        request = MarkVerifiedRequest()
        request.verification_id.value = "v:1"
        request.method = "eID"
        request.name = True
        request.birthdate = True
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/mark_verified",
            timeout=None,
        )

    async def test_verify_token(self):
        reply = VerifyTokenReply()
        reply.data.token_matches = True
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.verify_token(VerificationId("v:1"), "gazpacho"), True
        )

        request = VerifyTokenRequest()
        request.verification_id.value = "v:1"
        request.token = "gazpacho"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/verify_token",
            timeout=None,
        )

    async def test_get_contact_representative(self):
        message, address = get_address()
        reply = GetContactRepresentativeReply()
        reply.data.organization = "Jupiter Mining Corporation"
        reply.data.address.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.get_contact_representative(VerificationId("v:1")),
            ContactRepresentative(
                organization="Jupiter Mining Corporation",
                address=address,
            ),
        )

        request = GetContactRepresentativeRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/get_contact_representative",
            timeout=None,
        )

    async def test_set_contact_representative(self):
        message, address = get_address()
        reply = SetContactRepresentativeReply()
        self.client.mock.return_value = make_awaitable(reply)

        self.assertIsNone(
            await self.client.set_contact_representative(
                VerificationId("v:1"),
                ContactRepresentative(
                    organization="Jupiter Mining Corporation", address=address
                ),
            )
        )

        request = SetContactRepresentativeRequest()
        request.verification_id.value = "v:1"
        request.organization = "Jupiter Mining Corporation"
        request.address.CopyFrom(message)
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/set_contact_representative",
            timeout=None,
        )

==> fred_verifier-0.2.0-py3-none-any.whl: verifier/tests/test_client.py <==
from datetime import date, datetime, timedelta
from typing import Sequence, Tuple
from unittest import IsolatedAsyncioTestCase, TestCase
from unittest.mock import sentinel

import pytz
from aioitertools import iter as aiter
from fred_api.verification.common_types_pb2 import (
    Event as EventMessage,
    PlaceAddress,
    VerificationInfo,
    VerificationStatus as VerificationEnum,
)
from fred_api.verification.pagination_pb2 import PaginationReply
from fred_api.verification.service_verification_grpc_pb2 import (
    CancelReply,
    CancelRequest,
    CreateReply,
    CreateRequest,
    GetContactRepresentativeReply,
    GetContactRepresentativeRequest,
    GetMethodsReply,
    GetMethodsRequest,
    GetReply,
    GetRequest,
    ListContactDataVerificationsReply,
    ListContactDataVerificationsRequest,
    ListReply,
    ListRequest,
    SetContactRepresentativeReply,
    SetContactRepresentativeRequest,
    UseMethodReply,
    UseMethodRequest,
    VerifyContactDataReply,
    VerifyContactDataRequest,
    VerifyTokenReply,
    VerifyTokenRequest,
)
from frgal.utils import AsyncTestClientMixin, make_awaitable
from regal import Address, ContactId

from verifier.client import VerificationClient, VerificationDecoder
from verifier.common import (
    ContactData,
    ContactRepresentative,
    Event,
    PaginatedList,
    Pagination,
    Verification,
    VerificationId,
    VerificationResult,
    VerificationStatus,
)

DATETIME = datetime(2022, 1, 1, tzinfo=pytz.utc)


def get_event(
    event_datetime: datetime = DATETIME,
    type: str = "notification",
    method: str = None,
    details: dict = None,
) -> Tuple[EventMessage, Event]:
    """Get tuple of matching (gRPC message, Event instance)."""
    message = EventMessage()
    message.event_datetime.FromDatetime(event_datetime)
    if method:
        message.method = method
    message.type = type
    for key, value in (details or {}).items():
        message.details[key] = value

    event = Event(
        event_datetime=event_datetime, type=type, method=method, details=details or {}
    )
    return message, event


def get_verification(
    verification_id: str = "v:1",
    contact_id: str = "c:1",
    create_datetime: datetime = DATETIME,
    close_datetime: datetime = None,
    status: VerificationStatus = VerificationStatus.CREATED,
    next_events: Sequence[Event] = None,
    events: Sequence[Event] = None,
) -> Tuple[VerificationInfo, Verification]:
    """Get tuple of matching (gRPC message, Verification instance)."""
    message = VerificationInfo()
    message.verification_id.value = verification_id
    message.contact_id.value = contact_id
    message.create_datetime.FromDatetime(create_datetime)
    if close_datetime:
        message.close_datetime.FromDatetime(close_datetime)
    message.status = VerificationEnum.Value(status)

    for event in next_events or []:
        next_event_message, _ = get_event(**event.dict())
        subevent = message.next_events.add()
        subevent.CopyFrom(next_event_message)

    for event in events or []:
        event_message, _ = get_event(**event.dict())
        subevent = message.events.add()
        subevent.CopyFrom(event_message)

    verification = Verification(
        verification_id=verification_id,
        contact_id=contact_id,
        create_datetime=create_datetime,
        close_datetime=close_datetime,
        status=status,
        next_events=next_events or [],
        events=events or [],
    )

    return (message, verification)


def get_address(
    street: Sequence[str] = ("Deck 12", "Section B"),
    city: str = "Red Dwarf",
    state_or_province: str = "",
    postal_code: str = "123",
    country_code: str = "JMC",
) -> Tuple[PlaceAddress, Address]:
    message = PlaceAddress()
    message.street[:] = street
    message.city = city
    message.state_or_province = state_or_province
    message.postal_code = postal_code
    message.country_code = country_code

    address = Address(
        street=list(street),
        city=city,
        state_or_province=state_or_province,
        postal_code=postal_code,
        country_code=country_code,
    )

    return (message, address)


class TestVerificationClient(AsyncTestClientMixin, VerificationClient):
    """Test VerificationClient."""


class VerificationDecoderTest(TestCase):
    """Test VerificationDecoder."""

    def setUp(self):
        self.decoder = VerificationDecoder()

    def test_decode_event(self):
        message, event = get_event(details={"meta": "data"})
        self.assertEqual(self.decoder.decode(message), event)

    def test_decode_event_no_details(self):
        message, event = get_event()
        self.assertEqual(self.decoder.decode(message), event)

    def test_decode_verification_empty(self):
        message, verification = get_verification()
        self.assertEqual(self.decoder.decode(message), verification)

    def test_decode_verification_full(self):
        message, verification = get_verification(
            close_datetime=DATETIME + timedelta(days=1),
            status=VerificationStatus.CANCELED,
            next_events=[get_event(event_datetime=DATETIME + timedelta(days=1))[1]],
            events=[
                get_event(
                    event_datetime=DATETIME + timedelta(days=2),
                    method="eID",
                    details={"email": "kryten@example.org"},
                )[1]
            ],
        )
        self.assertEqual(self.decoder.decode(message), verification)

    def test_decode_pagination_empty(self):
        message = PaginationReply()
        self.assertEqual(self.decoder.decode(message), Pagination())

    def test_decode_pagination_full(self):
        message = PaginationReply()
        message.next_page_token = "gazpacho"
        message.items_left = 42
        self.assertEqual(
            self.decoder.decode(message),
            Pagination(next_page_token="gazpacho", items_left=42),
        )

    def test_decode_address(self):
        message, address = get_address()
        self.assertEqual(self.decoder.decode(message), address)


class VerificationClientTest(IsolatedAsyncioTestCase):
    """Test VerificationClient's methods."""

    def setUp(self):
        self.client = TestVerificationClient(netloc=sentinel.netloc)

    async def test_create(self):
        message, verification = get_verification()
        reply = CreateReply()
        reply.data.verification.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(await self.client.create(ContactId("c:1")), verification)

        request = CreateRequest()
        request.contact_id.value = "c:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/create",
            timeout=None,
        )

    async def test_cancel(self):
        reply = CancelReply()
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(await self.client.cancel(VerificationId("v:1")), None)

        request = CancelRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/cancel",
            timeout=None,
        )

    async def test_get(self):
        message, verification = get_verification()
        reply = GetReply()
        reply.data.verification.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(await self.client.get(VerificationId("v:1")), verification)

        request = GetRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/get",
            timeout=None,
        )

    async def test_list_empty(self):
        reply = ListReply()
        reply.data.verifications.extend([])
        self.client.mock.return_value = aiter([reply, ListReply()])

        self.assertEqual(
            await self.client.list(), PaginatedList(results=[], pagination=None)
        )

        request = ListRequest()
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/list",
            timeout=None,
        )

    async def test_list_full(self):
        message1, verification1 = get_verification(verification_id="v:1")
        message2, verification2 = get_verification(verification_id="v:2")
        reply1 = ListReply()
        reply1.data.verifications.add().CopyFrom(message1)
        reply1.data.pagination.next_page_token = "gazpacho!"
        reply1.data.pagination.items_left = 42
        reply2 = ListReply()
        reply2.data.verifications.add().CopyFrom(message2)
        self.client.mock.return_value = aiter([reply1, reply2])

        self.assertEqual(
            await self.client.list(
                contact_id=ContactId("RIMMER"),
                include_closed=True,
                page_token="gazpacho?",
                page_size=10,
            ),
            PaginatedList(
                results=[verification1, verification2],
                pagination=Pagination(next_page_token="gazpacho!", items_left=42),
            ),
        )

        request = ListRequest()
        request.contact_id.value = "RIMMER"
        request.include_closed = True
        request.pagination.page_token = "gazpacho?"
        request.pagination.page_size = 10
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/list",
            timeout=None,
        )

    async def test_get_methods(self):
        reply = GetMethodsReply()
        reply.data.methods[:] = ["eID", "letter"]
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.get_methods(VerificationId("v:1")), ["eID", "letter"]
        )

        request = GetMethodsRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/get_methods",
            timeout=None,
        )

    async def test_use_method(self):
        message, verification = get_verification()
        reply = UseMethodReply()
        reply.data.verification.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.use_method(VerificationId("v:1"), "eID"), verification
        )

        request = UseMethodRequest()
        request.verification_id.value = "v:1"
        request.method = "eID"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/use_method",
            timeout=None,
        )

    async def test_verify_token(self):
        reply = VerifyTokenReply()
        reply.data.token_matches = True
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.verify_token(VerificationId("v:1"), "gazpacho"), True
        )

        request = VerifyTokenRequest()
        request.verification_id.value = "v:1"
        request.token = "gazpacho"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/verify_token",
            timeout=None,
        )

    async def test_verify_contact_data_person(self):
        message, verification = get_verification(verification_id="v:1")
        data = ContactData(
            name="Arnold Rimmer",
            birthdate=date(2120, 1, 1),
        )
        reply = VerifyContactDataReply()
        reply.data.verification.CopyFrom(message)
        reply.data.result.verify_datetime.FromDatetime(
            datetime(2020, 1, 1, tzinfo=pytz.utc)
        )
        reply.data.result.method = "eID"
        reply.data.result.contact_history_id.value = "ch:1"
        reply.data.result.data.name = "Arnold Rimmer"
        reply.data.result.data.birthdate = "2120-01-01"
        reply.data.result.name = True
        reply.data.result.birthdate = False
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.verify_contact_data(VerificationId("v:1"), "eID", data),
            (
                verification,
                VerificationResult(
                    verify_datetime=datetime(2020, 1, 1, tzinfo=pytz.utc),
                    method="eID",
                    data=data,
                    contact_history_id="ch:1",
                    name=True,
                    birthdate=False,
                ),
            ),
        )

        request = VerifyContactDataRequest()
        request.verification_id.value = "v:1"
        request.method = "eID"
        request.data.name = "Arnold Rimmer"
        request.data.birthdate = "2120-01-01"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/verify_contact_data",
            timeout=None,
        )

    async def test_verify_contact_data_company(self):
        verif_message, verification = get_verification(verification_id="v:1")
        addr_message, address = get_address()
        data = ContactData(
            organization="Jupiter Mining Company",
            company_registration_number="123456",
            address=address,
        )
        reply = VerifyContactDataReply()
        reply.data.verification.CopyFrom(verif_message)
        reply.data.result.verify_datetime.FromDatetime(
            datetime(2020, 1, 1, tzinfo=pytz.utc)
        )
        reply.data.result.method = "ISDS"
        reply.data.result.contact_history_id.value = "ch:1"
        reply.data.result.data.organization = "Jupiter Mining Company"
        reply.data.result.data.company_registration_number = "123456"
        reply.data.result.data.address.CopyFrom(addr_message)
        reply.data.result.organization = True
        reply.data.result.company_registration_number = True
        reply.data.result.address = False
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.verify_contact_data(VerificationId("v:1"), "ISDS", data),
            (
                verification,
                VerificationResult(
                    verify_datetime=datetime(2020, 1, 1, tzinfo=pytz.utc),
                    method="ISDS",
                    data=data,
                    contact_history_id="ch:1",
                    organization=True,
                    company_registration_number=True,
                    address=False,
                ),
            ),
        )

        request = VerifyContactDataRequest()
        request.verification_id.value = "v:1"
        request.method = "ISDS"
        request.data.organization = "Jupiter Mining Company"
        request.data.company_registration_number = "123456"
        request.data.address.CopyFrom(addr_message)
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/verify_contact_data",
            timeout=None,
        )

    async def test_list_contact_data_verifications_empty(self):
        reply = ListContactDataVerificationsReply()
        reply.data.SetInParent()
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.list_contact_data_verifications(VerificationId("v:1")), []
        )

        request = ListContactDataVerificationsRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/list_contact_data_verifications",
            timeout=None,
        )

    async def test_list_contact_data_verifications(self):
        reply = ListContactDataVerificationsReply()
        result = reply.data.results.add()
        result.verify_datetime.FromDatetime(datetime(2020, 1, 1, tzinfo=pytz.utc))
        result.method = "ISDS"
        result.contact_history_id.value = "ch:1"
        result.data.organization = "Jupiter Mining Company"
        result.organization = True
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.list_contact_data_verifications(VerificationId("v:1")),
            [
                VerificationResult(
                    verify_datetime=datetime(2020, 1, 1, tzinfo=pytz.utc),
                    method="ISDS",
                    data=ContactData(organization="Jupiter Mining Company"),
                    contact_history_id="ch:1",
                    organization=True,
                )
            ],
        )

        request = ListContactDataVerificationsRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/list_contact_data_verifications",
            timeout=None,
        )

    async def test_get_contact_representative(self):
        message, address = get_address()
        reply = GetContactRepresentativeReply()
        reply.data.organization = "Jupiter Mining Corporation"
        reply.data.address.CopyFrom(message)
        self.client.mock.return_value = make_awaitable(reply)

        self.assertEqual(
            await self.client.get_contact_representative(VerificationId("v:1")),
            ContactRepresentative(
                organization="Jupiter Mining Corporation",
                address=address,
            ),
        )

        request = GetContactRepresentativeRequest()
        request.verification_id.value = "v:1"
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/get_contact_representative",
            timeout=None,
        )

    async def test_set_contact_representative(self):
        message, address = get_address()
        reply = SetContactRepresentativeReply()
        self.client.mock.return_value = make_awaitable(reply)

        self.assertIsNone(
            await self.client.set_contact_representative(
                VerificationId("v:1"),
                ContactRepresentative(
                    organization="Jupiter Mining Corporation", address=address
                ),
            )
        )

        request = SetContactRepresentativeRequest()
        request.verification_id.value = "v:1"
        request.organization = "Jupiter Mining Corporation"
        request.address.CopyFrom(message)
        self.client.mock.assert_called_once_with(
            request,
            method="/Fred.Verification.Api.Verification/set_contact_representative",
            timeout=None,
        )
